@@ -66,8 +66,7 @@ def aggregate(data, sorting="pca", tol=0.5): # , verbose=1
     fdim = data.shape[1]
     
     if sorting == "norm-mean" or sorting == "norm-orthant": 
-        c_data = data.copy()
-        sort_vals = np.linalg.norm(c_data, ord=2, axis=1)
+        sort_vals = np.linalg.norm(data, ord=2, axis=1)
         ind = np.argsort(sort_vals)
 
     elif sorting == "pca":
@@ -76,50 +75,46 @@ def aggregate(data, sorting="pca", tol=0.5): # , verbose=1
         # ind = np.argsort(sort_vals)
         
         # change to svd 
-        # data = data - data.mean(axis=0) -- already done in the clustering.fit_transform
-        c_data = data - data.mean(axis=0)
         if data.shape[1]>1:
-            gemm = get_blas_funcs("gemm", [c_data.T, c_data])
-            _, U1 = eigh(gemm(1, c_data.T, c_data), subset_by_index=[fdim-1, fdim-1])
-            sort_vals = c_data@U1.reshape(-1)
-            # U1, s1, _ = svds(c_data, k=1, return_singular_vectors="u")
-            # sort_vals = U1[:,0]*s1[0]
-            # print( U1, s1, _)
+            if fdim <= 3: # memory inefficient
+                gemm = get_blas_funcs("gemm", [data.T, data])
+                _, U1 = eigh(gemm(1, data.T, data), subset_by_index=[fdim-1, fdim-1])
+                sort_vals = data@U1.reshape(-1)
+            else:
+                U1, s1, _ = svds(data, k=1, return_singular_vectors="u")
+                sort_vals = U1[:,0]*s1[0]
+
         else:
-            sort_vals = c_data[:,0]
+            sort_vals = data[:,0]
+            
         sort_vals = sort_vals*np.sign(-sort_vals[0]) # flip to enforce deterministic output
         ind = np.argsort(sort_vals)
 
     else: # no sorting
-        sort_vals = np.zeros(len_ind) # useless, blankss
+        sort_vals = np.zeros(len_ind) 
         ind = np.arange(len_ind)
         
     lab = 0
     labels = [-1]*len_ind
     nr_dist = 0 
     
-    for i in range(len_ind): # tqdm（range(len_ind), disable=not verbose)
+    for i in range(len_ind): 
         sp = ind[i] # starting point
         if labels[sp] >= 0:
             continue
         else:
-            clustc = c_data[sp,:] 
+            clustc = data[sp,:] 
             labels[sp] = lab
             num_group = 1
 
-        for j in ind[i:]:
+        for j in ind[i+1:]:
             if labels[j] >= 0:
                 continue
 
-            # sort_val_c = sort_vals[sp]
-            # sort_val_j = sort_vals[j]
-
             if (sort_vals[j] - sort_vals[sp] > tol):
                 break       
 
-            # dist = np.sum((clustc - data[j,:])**2)    # slow
-
-            dat = clustc - c_data[j,:]
+            dat = clustc - data[j,:]
             dist = np.inner(dat, dat)
             nr_dist += 1
                 
@@ -127,23 +122,9 @@ def aggregate(data, sorting="pca", tol=0.5): # , verbose=1
                 num_group += 1
                 labels[j] = lab
 
-        splist.append([sp, sort_vals[sp], num_group])  # list of [ starting point index of current group, sorting key, and number of group elements ]
+        splist.append([sp, sort_vals[sp], num_group])  
+        # list of [ starting point index of current group, sorting key, and number of group elements ]
         lab += 1
 
-    # if verbose == 1:
-    #    print("aggregate {} groups".format(len(np.unique(labels))))
-
     return np.array(labels), splist, nr_dist
 
-
-
-
-
-# def calculate_group_centers(data, labels):
-#     agg_centers = list() 
-#     for c in set(labels):
-#         # indc = [i for i in range(data.shape[0]) if labels[i] == c]
-#         indc = np.where(labels==c)
-#         center = [-1, c] + np.mean(data[indc,:], axis=0).tolist()
-#         agg_centers.append( center )
-#     return agg_centers
